<?php

namespace Drupal\freecaster\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\Url;
use Drupal\freecaster\Services\FreecasterService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class SyncController.
 *
 * @package Drupal\freecaster\Controller
 */
final class SyncController extends ControllerBase {

  /**
   * Freecaster API service.
   *
   * @var \Drupal\freecaster\Services\FreecasterService
   */
  protected $freecaster;

  /**
   * State service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $messenger;

  /**
   * Date Formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * Messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $state;

  /**
   * {@inheritdoc}
   */
  public function __construct(FreecasterService $freecaster_service, StateInterface $state, DateFormatterInterface $date_formatter, MessengerInterface $messenger) {
    $this->freecaster = $freecaster_service;
    $this->state = $state;
    $this->dateFormatter = $date_formatter;
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('freecaster.api'),
      $container->get('state'),
      $container->get('date.formatter'),
      $container->get('messenger'),
    );
  }

  /**
   * Display the settings page for synchronisation.
   *
   * @return array
   *   Return a render array.
   */
  public function displayAdmin() {
    $last_update = $this->state->get('fc_last_sync');
    return [
      [
        '#type'   => 'item',
        '#title'  => $this->t('Last synchronisation'),
        '#markup' => $last_update ? $this->dateFormatter->format($last_update) : $this->t('Never'),
      ],
      [
        '#type'  => 'link',
        '#title' => $this->t('Link to the channels list'),
        '#url'   => URL::fromRoute(
          'entity.taxonomy_vocabulary.overview_form',
          ['taxonomy_vocabulary' => 'fc_video_channels'],
          ['attributes' => ['target' => '_blank']],
        ),
      ],

    ];
  }

  /**
   * Synchronise channels from Freecaster.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   Redirect to the sync settings page.
   */
  public function sync() {
    $this->messenger->addMessage('The channels have been synchronised with your Freecaster account');
    $this->freecaster->syncChannels();
    $this->state->set('fc_last_sync', time());
    return $this->redirect('freecaster.sync');
  }

}
