<?php

namespace Drupal\freecaster\Plugin\QueueWorker;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Queue\QueueWorkerBase;
use Drupal\Core\Queue\RequeueException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Worker plugin that update Freecaster media thumbnail when the video is ready.
 *
 * @QueueWorker(
 *   id = "fc_thumbnail_update",
 *   title = @Translation("Freecaster Thumbnail Update"),
 *   cron = {"time" = 30}
 * )
 */
final class FreecasterThumbnailUpdate extends QueueWorkerBase implements ContainerFactoryPluginInterface {

  /**
   * Entity Type service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Freecaster Service API.
   *
   * @var \Drupal\freecaster\Services\FreecasterService
   */
  protected $freecasterService;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    $entity_type_manager,
    $freecaster_service
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->freecasterService = $freecaster_service;
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('freecaster.api')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function processItem($data) {
    // Get Video without cache.
    $remote_video = $this->freecasterService->getVideo($data->video_id, TRUE);

    // Update the media thumbnail.
    if ($remote_video->status > -1) {
      $filter = [
        'bundle' => 'freecaster_video',
        'created' => $data->created,
        'field_video_fcid' => $data->video_id,
      ];
      $medias = $this->entityTypeManager->getStorage('media')->loadByProperties($filter);
      /** @var \Drupal\media\Entity\Media $media */
      $media = reset($medias);
      if ($media) {
        $media->updateQueuedThumbnail();
        $media->save();
      }
    }
    else {
      throw new RequeueException();
    }
  }

}
